package com.kexun.service.impl;

import com.alibaba.fastjson.JSON;
import com.kexun.constant.ReportConsts;
import com.kexun.entity.ReportInfo;
import com.kexun.enums.ReportStatusEnum;
import com.kexun.model.ReportStruct;
import com.kexun.model.Request;
import com.kexun.service.ReportInfoService;
import com.kexun.service.XmlFileHandlerService;
import lombok.extern.log4j.Log4j;
import org.apache.commons.io.FileUtils;
import org.dom4j.*;
import org.dom4j.io.SAXReader;
import org.omg.CORBA.IRObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.io.File;
import java.io.IOException;
import java.util.*;

/**
 * Description:征信报文文件处理类
 *
 * @author lgs
 * @date 2023/05/08
 */
@Service
@Log4j
public class XmlFileHandlerServiceImpl implements XmlFileHandlerService {

    @Value("${report.file.xmlSourcePath}")
    private String xmlSourcePath;

    @Value("${report.file.xmlBackPath}")
    private String xmlBackPath;

    @Autowired
    ReportInfoService reportInfoService;

    /**
     * 读取指定目录下报文文件
     */
    @Override
    public ReportStruct readXmlFile() {
        ReportStruct reportStruct = new ReportStruct();
        //循环读取该目录下文件
        if (!StringUtils.isEmpty(xmlSourcePath)) {
            File files = new File(xmlSourcePath);
            if (files.isDirectory()) {
                // 如果是目录，获取该目录下的内容集合
                File[] fileLists = files.listFiles();
                if (fileLists != null) {
                    //创建目录文件路径

                    for (File fileList : fileLists) {
                        //只处理xml文件
                        if (fileList.isFile() && fileList.getName().endsWith(".xml")) {
                            String name = fileList.getName();
                            log.info(name);
                            File file = new File(xmlSourcePath + name);
                            Map<String, Object> xmlMap = parse(file);
                            //处理公共信息
                            String fileName = file.getName();
                            xmlMap.put("fileName", fileName);
                            String xmlJson = JSON.toJSONString(xmlMap);
                            log.info("json串：" + xmlJson);
                            reportStruct = JSON.parseObject(xmlJson, ReportStruct.class);
                            saveReportInfo(reportStruct);
                            //将该目录下的文件进行移动，移动到指定的备份目录
                            File targetFile = new File(xmlBackPath+name);
                            try {
                                FileUtils.moveFile(file,targetFile);
                            } catch (IOException e) {
                                e.printStackTrace();
                            }
                        }
                    }
                } else {
                    log.info("该目录下文件为空");
                }

            } else {
                log.info("配置的征信报文源文件目录值不是目录，不进行处理");
            }
        } else {
            log.info("配置征信报文源文件目录为空，不进行处理");
        }
        return reportStruct;
    }

    /**
     * 保存到report_info信息表
     *
     * @param reportStruct
     */
    public void saveReportInfo(ReportStruct reportStruct) {
        //保存到report_info信息表
        ReportInfo reportInfo = new ReportInfo();
        //获取请求信息段
        Request request = reportStruct.getRequest();
        //业务申请ID
        reportInfo.setBusinessId(Long.valueOf(request.getId()));
        //业务申请编号
        reportInfo.setBusinessNum(request.getBusinessNum());
        //证件号码
        reportInfo.setCertificateNum(request.getCertificateNum());
        //客户编号
        reportInfo.setCustomerNum(request.getCustomerNum());
        //客户姓名
        reportInfo.setCustomerName(request.getCustomerName());
        //合作方编号
        reportInfo.setCoopBusinessNum(request.getCoopBusinessNum());
        //产品编号
        reportInfo.setProductNum(request.getProductNum());
        //扩展信息
        reportInfo.setExtend(request.getExtend());
        //报文体
        reportInfo.setBody(reportStruct.getResponseBody());
        //征信查询结果
        reportInfo.setResultCode(reportStruct.getResponseHeader().getResultCode());
        //征信查询结果描述
        reportInfo.setResultDesc(reportStruct.getResponseHeader().getResultMsg());
        //待解析状态
        String resultCode = reportStruct.getResponseHeader().getResultCode();
        String resultMsg = reportStruct.getResponseHeader().getResultMsg();
        if (ReportConsts.RETURN_CODE_0.equals(resultCode)) {
            if (ReportConsts.RESULT_MSG_SUCC_HAVE.equals(resultMsg)) {
                //查询成功，且有报告
                reportInfo.setStatus(ReportStatusEnum.NO_PRASE.getValue());
            } else if (ReportConsts.RESULT_MSG_SUCC_NOHAVE.equals(resultMsg)) {
                //查询成功，无报告
                reportInfo.setStatus(ReportStatusEnum.NO_REPORT.getValue());
            } else if (ReportConsts.RESULT_MSG_FAIL.equals(resultMsg)) {
                //查询失败
                reportInfo.setStatus(ReportStatusEnum.QUERY_FAIL.getValue());
            }
        } else if (ReportConsts.RETURN_CODE_1.equals(resultCode)) {
            reportInfo.setStatus(ReportStatusEnum.ERROR.getValue());
        }
        //文件名
        reportInfo.setFileName(reportStruct.getFileName());
        //保存报文信息
        reportInfoService.saveReportInfo(reportInfo);
    }


    /**
     * 解析xml节点内容
     *
     * @param file
     * @return
     */
    private Map<String, Object> parse(File file) {
        //创建SAXReader的对象
        SAXReader saxReader = new SAXReader();
        Map<String, Object> map = new HashMap<String, Object>();
        try {
            //通过reader对象的read方法加载xml文件，获取document对象
            Document document = saxReader.read(file);
            //通过document对象获取根节点
            Element rootElement = document.getRootElement();
            map = getAllElements(rootElement);
        } catch (DocumentException e) {
            e.printStackTrace();
        }
        return map;
    }

    /**
     * 获取xml节点要素
     *
     * @param rootElement
     * @return
     */
    public Map<String, Object> getAllElements(Element rootElement) {
        //存放xml节点元素的map集合
        Map<String, Object> mapEle = new HashMap<>();
        //1、处理request节点内容
        Map<String, Object> requestElements = getElements(rootElement, ReportConsts.REQUEST);
        //2、处理responseHeader节点内容
        Map<String, Object> responseHeaderElements = getElements(rootElement, ReportConsts.RESPONSE_HEADER);
        //3、处理responseBody节点内容
        Map<String, Object> responseBodyElements = getElements(rootElement, ReportConsts.RESPONSE_BODY);
        mapEle.putAll(requestElements);
        mapEle.putAll(responseHeaderElements);
        mapEle.putAll(responseBodyElements);
        return mapEle;
    }

    /**
     * 根据节点名称获取节点要素
     *
     * @param rootElement
     * @param key
     * @return
     */
    public HashMap<String, Object> getElements(Element rootElement, String key) {
        //存放xml节点元素的map集合
        HashMap<String, Object> mapEle = new HashMap<>();
        Element subElement = rootElement.element(key);
        Map<String, Object> requestMap = new HashMap<>(16);
        if (subElement.elements().size() > 0) {
            for (Element element : subElement.elements()) {
                String name = element.getName();
                String value = element.getText();
                requestMap.put(name, value);
            }
            mapEle.put(key, requestMap);
        } else {
            mapEle.put(key, subElement.getText());
        }

        return mapEle;
    }
}
